package net.knarcraft.permissionsigns.formatting;

import net.knarcraft.permissionsigns.PermissionSigns;
import net.knarcraft.permissionsigns.utility.FileHelper;
import org.bukkit.configuration.file.YamlConfiguration;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.UnsupportedEncodingException;
import java.util.HashMap;
import java.util.Map;
import java.util.logging.Level;

/**
 * A tool to get strings translated to the correct language
 */
public final class Translator {

    private static Map<TranslatableMessage, String> translatedMessages;
    private static Map<TranslatableMessage, String> backupTranslatedMessages;

    /**
     * Loads the languages used by this translator
     */
    public static void loadLanguages(String selectedLanguage) {
        backupTranslatedMessages = loadTranslatedMessages("en");
        translatedMessages = loadTranslatedMessages(selectedLanguage);
    }

    /**
     * Gets a translated version of the given translatable message
     *
     * @param translatableMessage <p>The message to translate</p>
     * @return <p>The translated message</p>
     */
    public static String getTranslatedMessage(TranslatableMessage translatableMessage) {
        if (translatedMessages == null) {
            return "Translated strings not loaded";
        }
        if (translatedMessages.containsKey(translatableMessage)) {
            return translatedMessages.get(translatableMessage);
        } else if (backupTranslatedMessages.containsKey(translatableMessage)) {
            return backupTranslatedMessages.get(translatableMessage);
        } else {
            return translatableMessage.toString();
        }
    }

    /**
     * Loads all translated messages for the given language
     *
     * @param language <p>The language chosen by the user</p>
     * @return <p>A mapping of all strings for the given language</p>
     */
    public static Map<TranslatableMessage, String> loadTranslatedMessages(String language) {
        Map<TranslatableMessage, String> translatedMessages = new HashMap<>();
        BufferedReader reader;
        try {
            reader = FileHelper.getBufferedReaderForInternalFile("/strings.yml");
        } catch (FileNotFoundException | UnsupportedEncodingException e) {
            PermissionSigns.getInstance().getLogger().log(Level.SEVERE, "Unable to load translated messages");
            return null;
        }
        YamlConfiguration configuration = YamlConfiguration.loadConfiguration(reader);

        for (TranslatableMessage message : TranslatableMessage.values()) {
            String translated = configuration.getString(language + "." + message.toString());
            if (translated != null) {
                translatedMessages.put(message, translated);
            }
        }
        return translatedMessages;
    }

}